@@ -1,90 +1,116 @@
-//----------------------------------------------------------------------------
-//  Copyright (C) 2008-2011  The IPython Development Team
-//
-//  Distributed under the terms of the BSD License.  The full license is in
-//  the file COPYING, distributed as part of this software.
-//----------------------------------------------------------------------------
-
-//============================================================================
-// CodeCell
-//============================================================================
-/**
- * An extendable module that provide base functionnality to create cell for notebook.
- * @module IPython
- * @namespace IPython
- * @submodule CodeCell
- */
-
-
-/* local util for codemirror */
-var posEq = function(a, b) {return a.line == b.line && a.ch == b.ch;};
-
+// Copyright (c) IPython Development Team.
+// Distributed under the terms of the Modified BSD License.
 /**
  *
- * function to delete until previous non blanking space character
- * or first multiple of 4 tabstop.
- * @private
+ *
+ * @module codecell
+ * @namespace codecell
+ * @class CodeCell
  */
-CodeMirror.commands.delSpaceToPrevTabStop = function(cm){
-    var from = cm.getCursor(true), to = cm.getCursor(false), sel = !posEq(from, to);
-    if (!posEq(from, to)) { cm.replaceRange("", from, to); return; }
-    var cur = cm.getCursor(), line = cm.getLine(cur.line);
-    var tabsize = cm.getOption('tabSize');
-    var chToPrevTabStop = cur.ch-(Math.ceil(cur.ch/tabsize)-1)*tabsize;
-    from = {ch:cur.ch-chToPrevTabStop,line:cur.line};
-    var select = cm.getRange(from,cur);
-    if( select.match(/^\ +$/) !== null){
-        cm.replaceRange("",from,cur);
-    } else {
-        cm.deleteH(-1,"char");
-    }
-};
 
 
-var IPython = (function (IPython) {
+define([
+    'base/js/namespace',
+    'jquery',
+    'base/js/utils',
+    'base/js/keyboard',
+    'services/config',
+    'notebook/js/cell',
+    'notebook/js/outputarea',
+    'notebook/js/completer',
+    'notebook/js/celltoolbar',
+    'codemirror/lib/codemirror',
+    'codemirror/mode/python/python',
+    'notebook/js/codemirror-ipython'
+], function(IPython,
+    $,
+    utils,
+    keyboard,
+    configmod,
+    cell,
+    outputarea,
+    completer,
+    celltoolbar,
+    CodeMirror,
+    cmpython,
+    cmip
+    ) {
     "use strict";
+    
+    var Cell = cell.Cell;
 
-    var utils = IPython.utils;
-    var keycodes = IPython.keyboard.keycodes;
+    /* local util for codemirror */
+    var posEq = function(a, b) {return a.line === b.line && a.ch === b.ch;};
 
     /**
-     * A Cell conceived to write code.
      *
-     * The kernel doesn't have to be set at creation time, in that case
-     * it will be null and set_kernel has to be called later.
-     * @class CodeCell
-     * @extends IPython.Cell
-     *
-     * @constructor
-     * @param {Object|null} kernel
-     * @param {object|undefined} [options]
-     *      @param [options.cm_config] {object} config to pass to CodeMirror
+     * function to delete until previous non blanking space character
+     * or first multiple of 4 tabstop.
+     * @private
      */
+    CodeMirror.commands.delSpaceToPrevTabStop = function(cm){
+        var from = cm.getCursor(true), to = cm.getCursor(false), sel = !posEq(from, to);
+        if (!posEq(from, to)) { cm.replaceRange("", from, to); return; }
+        var cur = cm.getCursor(), line = cm.getLine(cur.line);
+        var tabsize = cm.getOption('tabSize');
+        var chToPrevTabStop = cur.ch-(Math.ceil(cur.ch/tabsize)-1)*tabsize;
+        from = {ch:cur.ch-chToPrevTabStop,line:cur.line};
+        var select = cm.getRange(from,cur);
+        if( select.match(/^\ +$/) !== null){
+            cm.replaceRange("",from,cur);
+        } else {
+            cm.deleteH(-1,"char");
+        }
+    };
+
+    var keycodes = keyboard.keycodes;
+
     var CodeCell = function (kernel, options) {
+        /**
+         * Constructor
+         *
+         * A Cell conceived to write code.
+         *
+         * Parameters:
+         *  kernel: Kernel instance
+         *      The kernel doesn't have to be set at creation time, in that case
+         *      it will be null and set_kernel has to be called later.
+         *  options: dictionary
+         *      Dictionary of keyword arguments.
+         *          events: $(Events) instance 
+         *          config: dictionary
+         *          keyboard_manager: KeyboardManager instance 
+         *          notebook: Notebook instance
+         *          tooltip: Tooltip instance
+         */
         this.kernel = kernel || null;
+        this.notebook = options.notebook;
         this.collapsed = false;
+        this.events = options.events;
+        this.tooltip = options.tooltip;
+        this.config = options.config;
+        this.class_config = new configmod.ConfigWithDefaults(this.config,
+                                        CodeCell.config_defaults, 'CodeCell');
 
         // create all attributed in constructor function
         // even if null for V8 VM optimisation
         this.input_prompt_number = null;
         this.celltoolbar = null;
         this.output_area = null;
+
         this.last_msg_id = null;
         this.completer = null;
+        this.widget_views = [];
+        this._widgets_live = true;
 
-
-        var cm_overwrite_options  = {
-            onKeyEvent: $.proxy(this.handle_keyevent,this)
-        };
-
-        options = this.mergeopt(CodeCell, options, {cm_config:cm_overwrite_options});
-
-        IPython.Cell.apply(this,[options]);
+        Cell.apply(this,[{
+            config: $.extend({}, CodeCell.options_default), 
+            keyboard_manager: options.keyboard_manager, 
+            events: this.events}]);
 
         // Attributes we want to override in this subclass.
         this.cell_type = "code";
-
-        var that = this;
+        var that  = this;
         this.element.focusout(
             function() { that.auto_highlight(); }
         );
@@ -101,37 +127,51 @@ var IPython = (function (IPython) {
             },
             mode: 'ipython',
             theme: 'ipython',
-            matchBrackets: true,
-             // don't auto-close strings because of CodeMirror #2385
-            autoCloseBrackets: "()[]{}"
+            matchBrackets: true
         }
     };
 
+    CodeCell.config_defaults = {
+        highlight_modes : {
+            'magic_javascript'    :{'reg':[/^%%javascript/]},
+            'magic_perl'          :{'reg':[/^%%perl/]},
+            'magic_ruby'          :{'reg':[/^%%ruby/]},
+            'magic_python'        :{'reg':[/^%%python3?/]},
+            'magic_shell'         :{'reg':[/^%%bash/]},
+            'magic_r'             :{'reg':[/^%%R/]},
+            'magic_text/x-cython' :{'reg':[/^%%cython/]},
+        },
+    };
+
     CodeCell.msg_cells = {};
 
-    CodeCell.prototype = new IPython.Cell();
-
-    /**
-     * @method auto_highlight
-     */
-    CodeCell.prototype.auto_highlight = function () {
-        this._auto_highlight(IPython.config.cell_magic_highlight);
-    };
+    CodeCell.prototype = Object.create(Cell.prototype);
 
     /** @method create_element */
     CodeCell.prototype.create_element = function () {
-        IPython.Cell.prototype.create_element.apply(this, arguments);
+        Cell.prototype.create_element.apply(this, arguments);
+        var that = this;
 
-        var cell =  $('<div></div>').addClass('cell border-box-sizing code_cell');
+        var cell =  $('<div></div>').addClass('cell code_cell');
         cell.attr('tabindex','2');
 
         var input = $('<div></div>').addClass('input');
         var prompt = $('<div/>').addClass('prompt input_prompt');
         var inner_cell = $('<div/>').addClass('inner_cell');
-        this.celltoolbar = new IPython.CellToolbar(this);
+        this.celltoolbar = new celltoolbar.CellToolbar({
+            cell: this, 
+            notebook: this.notebook});
         inner_cell.append(this.celltoolbar.element);
         var input_area = $('<div/>').addClass('input_area');
-        this.code_mirror = CodeMirror(input_area.get(0), this.cm_config);
+        this.code_mirror = new CodeMirror(input_area.get(0), this.cm_config);
+        // In case of bugs that put the keyboard manager into an inconsistent state,
+        // ensure KM is enabled when CodeMirror is focused:
+        this.code_mirror.on('focus', function () {
+            if (that.keyboard_manager) {
+                that.keyboard_manager.enable();
+            }
+        });
+        this.code_mirror.on('keydown', $.proxy(this.handle_keyevent,this));
         $(this.code_mirror.getInputField()).attr("spellcheck", "false");
         inner_cell.append(input_area);
         input.append(prompt).append(inner_cell);
@@ -147,24 +187,98 @@ var IPython = (function (IPython) {
             .addClass('widget-subarea')
             .appendTo(widget_area);
         this.widget_subarea = widget_subarea;
+        var that = this;
         var widget_clear_buton = $('<button />')
             .addClass('close')
             .html('&times;')
             .click(function() {
-                widget_area.slideUp('', function(){ widget_subarea.html(''); });
-                })
+                widget_area.slideUp('', function(){ 
+                    for (var i = 0; i < that.widget_views.length; i++) {
+                        var view = that.widget_views[i];
+                        view.remove();
+
+                        // Remove widget live events.
+                        view.off('comm:live', that._widget_live);
+                        view.off('comm:dead', that._widget_dead);
+                    }
+                    that.widget_views = [];
+                    widget_subarea.html(''); 
+                });
+            })
             .appendTo(widget_prompt);
 
         var output = $('<div></div>');
         cell.append(input).append(widget_area).append(output);
         this.element = cell;
-        this.output_area = new IPython.OutputArea(output, true);
-        this.completer = new IPython.Completer(this);
+        this.output_area = new outputarea.OutputArea({
+            selector: output, 
+            prompt_area: true, 
+            events: this.events, 
+            keyboard_manager: this.keyboard_manager});
+        this.completer = new completer.Completer(this, this.events);
+    };
+
+    /**
+    *  Display a widget view in the cell.
+    */
+    CodeCell.prototype.display_widget_view = function(view_promise) {
+
+        // Display a dummy element
+        var dummy = $('<div/>');
+        this.widget_subarea.append(dummy);
+
+        // Display the view.
+        var that = this;
+        return view_promise.then(function(view) {
+            that.widget_area.show();
+            dummy.replaceWith(view.$el);
+            that.widget_views.push(view);
+
+            // Check the live state of the view's model.
+            if (view.model.comm_live) {
+                that._widget_live(view);
+            } else {
+                that._widget_dead(view);
+            }
+
+            // Listen to comm live events for the view.
+            view.on('comm:live', that._widget_live, that);
+            view.on('comm:dead', that._widget_dead, that);
+            return view;
+        });
+    };
+
+    /**
+     * Handles when a widget loses it's comm connection.
+     * @param  {WidgetView} view
+     */
+    CodeCell.prototype._widget_dead = function(view) {
+        if (this._widgets_live) {
+            this._widgets_live = false;
+            this.widget_area.addClass('connection-problems');
+        }
+
+    };
+
+    /**
+     * Handles when a widget is connected to a live comm.
+     * @param  {WidgetView} view
+     */
+    CodeCell.prototype._widget_live = function(view) {
+        if (!this._widgets_live) {
+            // Check that the other widgets are live too.  O(N) operation.
+            // Abort the function at the first dead widget found.
+            for (var i = 0; i < this.widget_views.length; i++) {
+                if (!this.widget_views[i].model.comm_live) return;
+            }
+            this._widgets_live = true;
+            this.widget_area.removeClass('connection-problems');
+        }
     };
 
     /** @method bind_events */
     CodeCell.prototype.bind_events = function () {
-        IPython.Cell.prototype.bind_events.apply(this);
+        Cell.prototype.bind_events.apply(this);
         var that = this;
 
         this.element.focusout(
@@ -180,14 +294,15 @@ var IPython = (function (IPython) {
      *  true = ignore, false = don't ignore.
      *  @method handle_codemirror_keyevent
      */
+
     CodeCell.prototype.handle_codemirror_keyevent = function (editor, event) {
 
         var that = this;
         // whatever key is pressed, first, cancel the tooltip request before
         // they are sent, and remove tooltip if any, except for tab again
         var tooltip_closed = null;
-        if (event.type === 'keydown' && event.which != keycodes.tab ) {
-            tooltip_closed = IPython.tooltip.remove_and_cancel_tooltip();
+        if (event.type === 'keydown' && event.which !== keycodes.tab ) {
+            tooltip_closed = this.tooltip.remove_and_cancel_tooltip();
         }
 
         var cur = editor.getCursor();
@@ -195,41 +310,52 @@ var IPython = (function (IPython) {
             this.auto_highlight();
         }
 
-        if (event.which === keycodes.down && event.type === 'keypress' && IPython.tooltip.time_before_tooltip >= 0) {
+        if (event.which === keycodes.down && event.type === 'keypress' && this.tooltip.time_before_tooltip >= 0) {
             // triger on keypress (!) otherwise inconsistent event.which depending on plateform
             // browser and keyboard layout !
             // Pressing '(' , request tooltip, don't forget to reappend it
             // The second argument says to hide the tooltip if the docstring
             // is actually empty
-            IPython.tooltip.pending(that, true);
+            this.tooltip.pending(that, true);
         } else if ( tooltip_closed && event.which === keycodes.esc && event.type === 'keydown') {
             // If tooltip is active, cancel it.  The call to
             // remove_and_cancel_tooltip above doesn't pass, force=true.
             // Because of this it won't actually close the tooltip
             // if it is in sticky mode. Thus, we have to check again if it is open
             // and close it with force=true.
-            if (!IPython.tooltip._hidden) {
-                IPython.tooltip.remove_and_cancel_tooltip(true);
+            if (!this.tooltip._hidden) {
+                this.tooltip.remove_and_cancel_tooltip(true);
             }
             // If we closed the tooltip, don't let CM or the global handlers
             // handle this event.
-            event.stop();
+            event.codemirrorIgnore = true;
+            event._ipkmIgnore = true;
+            event.preventDefault();
             return true;
         } else if (event.keyCode === keycodes.tab && event.type === 'keydown' && event.shiftKey) {
-                if (editor.somethingSelected()){
+                if (editor.somethingSelected() || editor.getSelections().length !== 1){
                     var anchor = editor.getCursor("anchor");
                     var head = editor.getCursor("head");
-                    if( anchor.line != head.line){
+                    if( anchor.line !== head.line){
                         return false;
                     }
                 }
-                IPython.tooltip.request(that);
-                event.stop();
+                var pre_cursor = editor.getRange({line:cur.line,ch:0},cur);
+                if (pre_cursor.trim() === "") {
+                    // Don't show tooltip if the part of the line before the cursor
+                    // is empty.  In this case, let CodeMirror handle indentation.
+                    return false;
+                } 
+                this.tooltip.request(that);
+                event.codemirrorIgnore = true;
+                event.preventDefault();
                 return true;
-        } else if (event.keyCode === keycodes.tab && event.type == 'keydown') {
+        } else if (event.keyCode === keycodes.tab && event.type === 'keydown') {
             // Tab completion.
-            IPython.tooltip.remove_and_cancel_tooltip();
-            if (editor.somethingSelected()) {
+            this.tooltip.remove_and_cancel_tooltip();
+
+            // completion does not work on multicursor, it might be possible though in some cases
+            if (editor.somethingSelected() || editor.getSelections().length > 1) {
                 return false;
             }
             var pre_cursor = editor.getRange({line:cur.line,ch:0},cur);
@@ -238,7 +364,8 @@ var IPython = (function (IPython) {
                 // is empty.  In this case, let CodeMirror handle indentation.
                 return false;
             } else {
-                event.stop();
+                event.codemirrorIgnore = true;
+                event.preventDefault();
                 this.completer.startCompletion();
                 return true;
             }
@@ -246,7 +373,7 @@ var IPython = (function (IPython) {
         
         // keyboard event wasn't one of those unique to code cells, let's see
         // if it's one of the generic ones (i.e. check edit mode shortcuts)
-        return IPython.Cell.prototype.handle_codemirror_keyevent.apply(this, [editor, event]);
+        return Cell.prototype.handle_codemirror_keyevent.apply(this, [editor, event]);
     };
 
     // Kernel related calls.
@@ -259,28 +386,55 @@ var IPython = (function (IPython) {
      * Execute current code cell to the kernel
      * @method execute
      */
-    CodeCell.prototype.execute = function () {
-        this.output_area.clear_output();
-        
+    CodeCell.prototype.execute = function (stop_on_error) {
+        if (!this.kernel || !this.kernel.is_connected()) {
+            console.log("Can't execute, kernel is not connected.");
+            return;
+        }
+
+        this.output_area.clear_output(false, true);
+
+        if (stop_on_error === undefined) {
+            stop_on_error = true;
+        }
+
         // Clear widget area
+        for (var i = 0; i < this.widget_views.length; i++) {
+            var view = this.widget_views[i];
+            view.remove();
+
+            // Remove widget live events.
+            view.off('comm:live', this._widget_live);
+            view.off('comm:dead', this._widget_dead);
+        }
+        this.widget_views = [];
         this.widget_subarea.html('');
         this.widget_subarea.height('');
         this.widget_area.height('');
         this.widget_area.hide();
-
-        this.set_input_prompt('*');
-        this.element.addClass("running");
-        if (this.last_msg_id) {
-            this.kernel.clear_callbacks_for_msg(this.last_msg_id);
-        }
-        var callbacks = this.get_callbacks();
         
         var old_msg_id = this.last_msg_id;
-        this.last_msg_id = this.kernel.execute(this.get_text(), callbacks, {silent: false, store_history: true});
+
         if (old_msg_id) {
-            delete CodeCell.msg_cells[old_msg_id];
+            this.kernel.clear_callbacks_for_msg(old_msg_id);
+            if (old_msg_id) {
+                delete CodeCell.msg_cells[old_msg_id];
+            }
         }
+        if (this.get_text().trim().length === 0) {
+            // nothing to do
+            this.set_input_prompt(null);
+            return;
+        }
+        this.set_input_prompt('*');
+        this.element.addClass("running");
+        var callbacks = this.get_callbacks();
+        
+        this.last_msg_id = this.kernel.execute(this.get_text(), callbacks, {silent: false, store_history: true,
+            stop_on_error : stop_on_error});
         CodeCell.msg_cells[this.last_msg_id] = this;
+        this.render();
+        this.events.trigger('execute.CodeCell', {cell: this});
     };
     
     /**
@@ -288,6 +442,7 @@ var IPython = (function (IPython) {
      * @method get_callbacks
      */
     CodeCell.prototype.get_callbacks = function () {
+        var that = this;
         return {
             shell : {
                 reply : $.proxy(this._handle_execute_reply, this),
@@ -297,15 +452,19 @@ var IPython = (function (IPython) {
                 }
             },
             iopub : {
-                output : $.proxy(this.output_area.handle_output, this.output_area),
-                clear_output : $.proxy(this.output_area.handle_clear_output, this.output_area),
+                output : function() { 
+                    that.output_area.handle_output.apply(that.output_area, arguments);
+                }, 
+                clear_output : function() { 
+                    that.output_area.handle_clear_output.apply(that.output_area, arguments);
+                }, 
             },
             input : $.proxy(this._handle_input_request, this)
         };
     };
     
     CodeCell.prototype._open_with_pager = function (payload) {
-        $([IPython.events]).trigger('open_with_text.Pager', payload);
+        this.events.trigger('open_with_text.Pager', payload);
     };
 
     /**
@@ -315,7 +474,7 @@ var IPython = (function (IPython) {
     CodeCell.prototype._handle_execute_reply = function (msg) {
         this.set_input_prompt(msg.content.execution_count);
         this.element.removeClass("running");
-        $([IPython.events]).trigger('set_dirty.Notebook', {value: true});
+        this.events.trigger('set_dirty.Notebook', {value: true});
     };
 
     /**
@@ -323,8 +482,8 @@ var IPython = (function (IPython) {
      * @private
      */
     CodeCell.prototype._handle_set_next_input = function (payload) {
-        var data = {'cell': this, 'text': payload.text};
-        $([IPython.events]).trigger('set_next_input.Notebook', data);
+        var data = {'cell': this, 'text': payload.text, replace: payload.replace};
+        this.events.trigger('set_next_input.Notebook', data);
     };
 
     /**
@@ -339,7 +498,7 @@ var IPython = (function (IPython) {
     // Basic cell manipulation.
 
     CodeCell.prototype.select = function () {
-        var cont = IPython.Cell.prototype.select.apply(this);
+        var cont = Cell.prototype.select.apply(this);
         if (cont) {
             this.code_mirror.refresh();
             this.auto_highlight();
@@ -348,16 +507,11 @@ var IPython = (function (IPython) {
     };
 
     CodeCell.prototype.render = function () {
-        var cont = IPython.Cell.prototype.render.apply(this);
+        var cont = Cell.prototype.render.apply(this);
         // Always execute, even if we are already in the rendered state
         return cont;
     };
     
-    CodeCell.prototype.unrender = function () {
-        // CodeCell is always rendered
-        return false;
-    };
-
     CodeCell.prototype.select_all = function () {
         var start = {line: 0, ch: 0};
         var nlines = this.code_mirror.lineCount();
@@ -368,13 +522,11 @@ var IPython = (function (IPython) {
 
 
     CodeCell.prototype.collapse_output = function () {
-        this.collapsed = true;
         this.output_area.collapse();
     };
 
 
     CodeCell.prototype.expand_output = function () {
-        this.collapsed = false;
         this.output_area.expand();
         this.output_area.unscroll_area();
     };
@@ -385,7 +537,6 @@ var IPython = (function (IPython) {
     };
 
     CodeCell.prototype.toggle_output = function () {
-        this.collapsed = Boolean(1 - this.collapsed);
         this.output_area.toggle_output();
     };
 
@@ -451,45 +602,40 @@ var IPython = (function (IPython) {
     // JSON serialization
 
     CodeCell.prototype.fromJSON = function (data) {
-        IPython.Cell.prototype.fromJSON.apply(this, arguments);
+        Cell.prototype.fromJSON.apply(this, arguments);
         if (data.cell_type === 'code') {
-            if (data.input !== undefined) {
-                this.set_text(data.input);
+            if (data.source !== undefined) {
+                this.set_text(data.source);
                 // make this value the starting point, so that we can only undo
                 // to this state, instead of a blank cell
                 this.code_mirror.clearHistory();
                 this.auto_highlight();
             }
-            if (data.prompt_number !== undefined) {
-                this.set_input_prompt(data.prompt_number);
-            } else {
-                this.set_input_prompt();
-            }
-            this.output_area.trusted = data.trusted || false;
-            this.output_area.fromJSON(data.outputs);
-            if (data.collapsed !== undefined) {
-                if (data.collapsed) {
-                    this.collapse_output();
-                } else {
-                    this.expand_output();
-                }
-            }
+            this.set_input_prompt(data.execution_count);
+            this.output_area.trusted = data.metadata.trusted || false;
+            this.output_area.fromJSON(data.outputs, data.metadata);
         }
     };
 
 
     CodeCell.prototype.toJSON = function () {
-        var data = IPython.Cell.prototype.toJSON.apply(this);
-        data.input = this.get_text();
+        var data = Cell.prototype.toJSON.apply(this);
+        data.source = this.get_text();
         // is finite protect against undefined and '*' value
         if (isFinite(this.input_prompt_number)) {
-            data.prompt_number = this.input_prompt_number;
+            data.execution_count = this.input_prompt_number;
+        } else {
+            data.execution_count = null;
         }
         var outputs = this.output_area.toJSON();
         data.outputs = outputs;
-        data.language = 'python';
-        data.trusted = this.output_area.trusted;
-        data.collapsed = this.collapsed;
+        data.metadata.trusted = this.output_area.trusted;
+        data.metadata.collapsed = this.output_area.collapsed;
+        if (this.output_area.scroll_state === 'auto') {
+            delete data.metadata.scrolled;
+        } else {
+            data.metadata.scrolled = this.output_area.scroll_state;
+        }
         return data;
     };
 
@@ -499,11 +645,11 @@ var IPython = (function (IPython) {
      * @return is the action being taken
      */
     CodeCell.prototype.unselect = function () {
-        var cont = IPython.Cell.prototype.unselect.apply(this);
+        var cont = Cell.prototype.unselect.apply(this);
         if (cont) {
             // When a code cell is usnelected, make sure that the corresponding
             // tooltip and completer to that cell is closed.
-            IPython.tooltip.remove_and_cancel_tooltip(true);
+            this.tooltip.remove_and_cancel_tooltip(true);
             if (this.completer !== null) {
                 this.completer.close();
             }
@@ -511,7 +657,8 @@ var IPython = (function (IPython) {
         return cont;
     };
 
+    // Backwards compatability.
     IPython.CodeCell = CodeCell;
 
-    return IPython;
-}(IPython));
+    return {'CodeCell': CodeCell};
+});
