@@ -6,34 +6,52 @@ test suite.
 
 """
 
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2009-2011  The IPython Development Team
-#
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
-#-----------------------------------------------------------------------------
+# Copyright (c) IPython Development Team.
+# Distributed under the terms of the Modified BSD License.
 
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
 from __future__ import print_function
 
 import argparse
 import json
 import multiprocessing.pool
 import os
+import stat
+import re
+import requests
 import shutil
 import signal
 import sys
 import subprocess
 import time
 
-from .iptest import have, test_group_names as py_test_group_names, test_sections, StreamCapturer
+from .iptest import (
+    have, test_group_names as py_test_group_names, test_sections, StreamCapturer,
+    test_for,
+)
 from IPython.utils.path import compress_user
 from IPython.utils.py3compat import bytes_to_str
 from IPython.utils.sysinfo import get_sys_info
 from IPython.utils.tempdir import TemporaryDirectory
+from IPython.utils.text import strip_ansi
 
+try:
+    # Python >= 3.3
+    from subprocess import TimeoutExpired
+    def popen_wait(p, timeout):
+        return p.wait(timeout)
+except ImportError:
+    class TimeoutExpired(Exception):
+        pass
+    def popen_wait(p, timeout):
+        """backport of Popen.wait from Python 3"""
+        for i in range(int(10 * timeout)):
+            if p.poll() is not None:
+                return
+            time.sleep(0.1)
+        if p.poll() is None:
+            raise TimeoutExpired
+
+NOTEBOOK_SHUTDOWN_TIMEOUT = 10
 
 class TestController(object):
     """Run tests in a subprocess
@@ -64,18 +82,24 @@ class TestController(object):
         """
         pass
 
-    def launch(self, buffer_output=False):
+    def launch(self, buffer_output=False, capture_output=False):
         # print('*** ENV:', self.env)  # dbg
         # print('*** CMD:', self.cmd)  # dbg
         env = os.environ.copy()
         env.update(self.env)
-        output = subprocess.PIPE if buffer_output else None
-        stdout = subprocess.STDOUT if buffer_output else None
-        self.process = subprocess.Popen(self.cmd, stdout=output,
-                stderr=stdout, env=env)
+        if buffer_output:
+            capture_output = True
+        self.stdout_capturer = c = StreamCapturer(echo=not buffer_output)
+        c.start()
+        stdout = c.writefd if capture_output else None
+        stderr = subprocess.STDOUT if capture_output else None
+        self.process = subprocess.Popen(self.cmd, stdout=stdout,
+                stderr=stderr, env=env)
 
     def wait(self):
-        self.stdout, _ = self.process.communicate()
+        self.process.wait()
+        self.stdout_capturer.halt()
+        self.stdout = self.stdout_capturer.get_buffer()
         return self.process.returncode
 
     def print_extra_info(self):
@@ -122,6 +146,7 @@ class TestController(object):
 
     __del__ = cleanup
 
+
 class PyTestController(TestController):
     """Run Python tests using IPython.testing.iptest"""
     #: str, Python command to execute in subprocess
@@ -145,6 +170,21 @@ class PyTestController(TestController):
         self.env['IPTEST_WORKING_DIR'] = workingdir.name
         # This means we won't get odd effects from our own matplotlib config
         self.env['MPLCONFIGDIR'] = workingdir.name
+        # For security reasons (http://bugs.python.org/issue16202), use
+        # a temporary directory to which other users have no access.
+        self.env['TMPDIR'] = workingdir.name
+
+        # Add a non-accessible directory to PATH (see gh-7053)
+        noaccess = os.path.join(self.workingdir.name, "_no_access_")
+        self.noaccess = noaccess
+        os.mkdir(noaccess, 0)
+
+        PATH = os.environ.get('PATH', '')
+        if PATH:
+            PATH = noaccess + os.pathsep + PATH
+        else:
+            PATH = noaccess
+        self.env['PATH'] = PATH
 
         # From options:
         if self.options.xunit:
@@ -154,6 +194,14 @@ class PyTestController(TestController):
         self.env['IPTEST_SUBPROC_STREAMS'] = self.options.subproc_streams
         self.cmd.extend(self.options.extra_args)
 
+    def cleanup(self):
+        """
+        Make the non-accessible directory created in setup() accessible
+        again, otherwise deleting the workingdir will fail.
+        """
+        os.chmod(self.noaccess, stat.S_IRWXU)
+        TestController.cleanup(self)
+
     @property
     def will_run(self):
         try:
@@ -188,6 +236,7 @@ class PyTestController(TestController):
         self.cmd[2] = self.pycmd
         super(PyTestController, self).launch(buffer_output=buffer_output)
 
+
 js_prefix = 'js/'
 
 def get_js_test_dir():
@@ -197,19 +246,27 @@ def get_js_test_dir():
 def all_js_groups():
     import glob
     test_dir = get_js_test_dir()
-    all_subdirs = glob.glob(test_dir + '*/')
-    return [js_prefix+os.path.relpath(x, test_dir) for x in all_subdirs if os.path.relpath(x, test_dir) != '__pycache__']
+    all_subdirs = glob.glob(test_dir + '[!_]*/')
+    return [js_prefix+os.path.relpath(x, test_dir) for x in all_subdirs]
 
 class JSController(TestController):
     """Run CasperJS tests """
-    def __init__(self, section):
+    
+    requirements =  ['zmq', 'tornado', 'jinja2', 'casperjs', 'sqlite3',
+                     'jsonschema']
+
+    def __init__(self, section, xunit=True, engine='phantomjs', url=None):
         """Create new test runner."""
         TestController.__init__(self)
+        self.engine = engine
         self.section = section
+        self.xunit = xunit
+        self.url = url
+        self.slimer_failure = re.compile('^FAIL.*', flags=re.MULTILINE)
         js_test_dir = get_js_test_dir()
         includes = '--includes=' + os.path.join(js_test_dir,'util.js')
         test_cases = os.path.join(js_test_dir, self.section[len(js_prefix):])
-        self.cmd = ['casperjs', 'test', includes, test_cases]
+        self.cmd = ['casperjs', 'test', includes, test_cases, '--engine=%s' % self.engine]
 
     def setup(self):
         self.ipydir = TemporaryDirectory()
@@ -218,22 +275,65 @@ class JSController(TestController):
         self.dirs.append(self.nbdir)
         os.makedirs(os.path.join(self.nbdir.name, os.path.join(u'sub ∂ir1', u'sub ∂ir 1a')))
         os.makedirs(os.path.join(self.nbdir.name, os.path.join(u'sub ∂ir2', u'sub ∂ir 1b')))
-        
-        # start the ipython notebook, so we get the port number
-        self.server_port = 0
-        self._init_server()
-        if self.server_port:
-            self.cmd.append("--port=%i" % self.server_port)
+
+        if self.xunit:
+            self.add_xunit()
+
+        # If a url was specified, use that for the testing.
+        if self.url:
+            try:
+                alive = requests.get(self.url).status_code == 200
+            except:
+                alive = False
+
+            if alive:
+                self.cmd.append("--url=%s" % self.url)
+            else:
+                raise Exception('Could not reach "%s".' % self.url)
         else:
-            # don't launch tests if the server didn't start
-            self.cmd = [sys.executable, '-c', 'raise SystemExit(1)']
-    
+            # start the ipython notebook, so we get the port number
+            self.server_port = 0
+            self._init_server()
+            if self.server_port:
+                self.cmd.append("--port=%i" % self.server_port)
+            else:
+                # don't launch tests if the server didn't start
+                self.cmd = [sys.executable, '-c', 'raise SystemExit(1)']
+
+    def add_xunit(self):
+        xunit_file = os.path.abspath(self.section.replace('/','.') + '.xunit.xml')
+        self.cmd.append('--xunit=%s' % xunit_file)
+
+    def launch(self, buffer_output):
+        # If the engine is SlimerJS, we need to buffer the output because
+        # SlimerJS does not support exit codes, so CasperJS always returns 0.
+        if self.engine == 'slimerjs' and not buffer_output:
+            return super(JSController, self).launch(capture_output=True)
+
+        else:
+            return super(JSController, self).launch(buffer_output=buffer_output)
+
+    def wait(self, *pargs, **kwargs):
+        """Wait for the JSController to finish"""
+        ret = super(JSController, self).wait(*pargs, **kwargs)
+        # If this is a SlimerJS controller, check the captured stdout for
+        # errors.  Otherwise, just return the return code.
+        if self.engine == 'slimerjs':
+            stdout = bytes_to_str(self.stdout)
+            if ret != 0:
+                # This could still happen e.g. if it's stopped by SIGINT
+                return ret
+            return bool(self.slimer_failure.search(strip_ansi(stdout)))
+        else:
+            return ret
+
     def print_extra_info(self):
         print("Running tests with notebook directory %r" % self.nbdir.name)
 
     @property
     def will_run(self):
-        return all(have[a] for a in ['zmq', 'tornado', 'jinja2', 'casperjs', 'sqlite3'])
+        should_run = all(have[a] for a in self.requirements + [self.engine])
+        return should_run
 
     def _init_server(self):
         "Start the notebook server in a separate process"
@@ -249,7 +349,15 @@ class JSController(TestController):
             command.append('--KernelManager.transport=ipc')
         self.stream_capturer = c = StreamCapturer()
         c.start()
-        self.server = subprocess.Popen(command, stdout=c.writefd, stderr=subprocess.STDOUT)
+        env = os.environ.copy()
+        if self.engine == 'phantomjs':
+            env['IPYTHON_ALLOW_DRAFT_WEBSOCKETS_FOR_PHANTOMJS'] = '1'
+        self.server = subprocess.Popen(command,
+            stdout=c.writefd,
+            stderr=subprocess.STDOUT,
+            cwd=self.nbdir.name,
+            env=env,
+        )
         self.server_info_file = os.path.join(self.ipydir.name,
             'profile_default', 'security', 'nbserver-%i.json' % self.server.pid
         )
@@ -261,8 +369,14 @@ class JSController(TestController):
             if self.server.poll() is not None:
                 return self._failed_to_start()
             if os.path.exists(self.server_info_file):
-                self._load_server_info()
-                return
+                try:
+                    self._load_server_info()
+                except ValueError:
+                    # If the server is halfway through writing the file, we may
+                    # get invalid JSON; it should be ready next iteration.
+                    pass
+                else:
+                    return
             time.sleep(0.1)
         print("Notebook server-info file never arrived: %s" % self.server_info_file,
             file=sys.stderr
@@ -282,13 +396,34 @@ class JSController(TestController):
         self.server_port = info['port']
 
     def cleanup(self):
-        try:
-            self.server.terminate()
-        except OSError:
-            # already dead
-            pass
-        self.server.wait()
-        self.stream_capturer.halt()
+        if hasattr(self, 'server'):
+            try:
+                self.server.terminate()
+            except OSError:
+                # already dead
+                pass
+            # wait 10s for the server to shutdown
+            try:
+                popen_wait(self.server, NOTEBOOK_SHUTDOWN_TIMEOUT)
+            except TimeoutExpired:
+                # server didn't terminate, kill it
+                try:
+                    print("Failed to terminate notebook server, killing it.",
+                        file=sys.stderr
+                    )
+                    self.server.kill()
+                except OSError:
+                    # already dead
+                    pass
+            # wait another 10s
+            try:
+                popen_wait(self.server, NOTEBOOK_SHUTDOWN_TIMEOUT)
+            except TimeoutExpired:
+                print("Notebook server still running (%s)" % self.server_info_file,
+                    file=sys.stderr
+                )
+              
+            self.stream_capturer.halt()
         TestController.cleanup(self)
 
 
@@ -296,21 +431,22 @@ def prepare_controllers(options):
     """Returns two lists of TestController instances, those to run, and those
     not to run."""
     testgroups = options.testgroups
-
     if testgroups:
-        py_testgroups = [g for g in testgroups if (g in py_test_group_names) \
-                                                or g.startswith('IPython.')]
         if 'js' in testgroups:
             js_testgroups = all_js_groups()
         else:
-            js_testgroups = [g for g in testgroups if g not in py_testgroups]
+            js_testgroups = [g for g in testgroups if g.startswith(js_prefix)]
+        py_testgroups = [g for g in testgroups if not g.startswith('js')]
     else:
         py_testgroups = py_test_group_names
-        js_testgroups = all_js_groups()
         if not options.all:
+            js_testgroups = []
             test_sections['parallel'].enabled = False
+        else:
+            js_testgroups = all_js_groups()
 
-    c_js = [JSController(name) for name in js_testgroups]
+    engine = 'slimerjs' if options.slimerjs else 'phantomjs'
+    c_js = [JSController(name, xunit=options.xunit, engine=engine, url=options.url) for name in js_testgroups]
     c_py = [PyTestController(name, options) for name in py_testgroups]
 
     controllers = c_py + c_js
@@ -415,6 +551,12 @@ def run_iptestall(options):
       Include slow tests, like IPython.parallel. By default, these tests aren't
       run.
 
+    slimerjs : bool
+      Use slimerjs if it's installed instead of phantomjs for casperjs tests.
+
+    url : unicode
+      Address:port to use when running the JS tests.
+
     xunit : bool
       Produce Xunit XML output. This is written to multiple foo.xunit.xml files.
 
@@ -494,7 +636,7 @@ def run_iptestall(options):
         print()
 
     if options.coverage:
-        from coverage import coverage
+        from coverage import coverage, CoverageException
         cov = coverage(data_file='.coverage')
         cov.combine()
         cov.save()
@@ -529,7 +671,12 @@ def run_iptestall(options):
 
         # Coverage XML report
         elif options.coverage == 'xml':
-            cov.xml_report(outfile='ipy_coverage.xml')
+            try:
+                cov.xml_report(outfile='ipy_coverage.xml')
+            except CoverageException as e:
+                print('Generating coverage report failed. Are you running javascript tests only?')
+                import traceback
+                traceback.print_exc()
 
     if failed:
         # Ensure that our exit code indicates failure
@@ -541,6 +688,9 @@ argparser.add_argument('testgroups', nargs='*',
                     'all tests.')
 argparser.add_argument('--all', action='store_true',
                     help='Include slow tests not run by default.')
+argparser.add_argument('--slimerjs', action='store_true',
+                    help="Use slimerjs if it's installed instead of phantomjs for casperjs tests.")
+argparser.add_argument('--url', help="URL to use for the JS tests.")
 argparser.add_argument('-j', '--fast', nargs='?', const=None, default=1, type=int,
                     help='Run test sections in parallel. This starts as many '
                     'processes as you have cores, or you can specify a number.')
